"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.contextRules = exports.isInRoleMethod = exports.isInContext = exports.isContext = exports.isRoleMethod = exports.currentFunction = exports.createRule = void 0;
const utils_1 = require("@typescript-eslint/utils");
const context_1 = require("./context");
const debug_1 = __importDefault(require("./debug"));
const d = (0, debug_1.default)("rules");
exports.createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://example.com/rule/${name}`);
// TODO: Role name checking rule
// TODO: Role splitting config setting
const isContextRegexp = /\W*@DCI-context\b/i;
const _currentContext = [];
const _currentRoleMethod = [];
let _currentFunction = null;
const currentContext = () => _currentContext[_currentContext.length - 1];
const currentRoleMethod = () => _currentRoleMethod[_currentRoleMethod.length - 1];
const currentFunction = () => _currentFunction;
exports.currentFunction = currentFunction;
const isRoleMethod = (func) => func && currentRoleMethod()?.func === func ? currentRoleMethod() : undefined;
exports.isRoleMethod = isRoleMethod;
const isContext = (func) => func && currentContext()?.func === func ? currentContext() : undefined;
exports.isContext = isContext;
const isInContext = () => currentContext();
exports.isInContext = isInContext;
const isInRoleMethod = () => currentRoleMethod();
exports.isInRoleMethod = isInRoleMethod;
const enterFunction = (func, context) => {
    _currentFunction = func;
    const currentCtx = currentContext();
    // Skip if we're already in the same context
    if (currentCtx?.func == func)
        return;
    let roleMethod;
    if (currentCtx) {
        // Check if entering a RoleMethod
        roleMethod = currentCtx.roleMethodFromFunc(func);
        if (roleMethod && currentRoleMethod()?.func != func) {
            d(`Entering RoleMethod ${roleMethod.role}.${roleMethod.method}`);
            _currentRoleMethod.push(roleMethod);
        }
    }
    if (!roleMethod) {
        const source = context.getSourceCode();
        const comments = currentCtx
            ? source.getCommentsBefore(func)
            : [func, ...context.getAncestors()]
                .filter((n) => n.parent)
                .flatMap((n) => source.getCommentsBefore(n));
        const commentStr = comments.map((c) => c.value).join(" ");
        const hasContextComment = commentStr.match(isContextRegexp);
        if (hasContextComment) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            const name = func.id?.name ?? func.parent?.id?.name;
            d("Entering Context " + name);
            _currentContext.push(new context_1.Context(context, name, func));
        }
    }
};
const exitFunction = (func) => {
    _currentFunction = null;
    const currentCtx = currentContext();
    if (currentCtx?.func == func) {
        d("Exiting Context " + currentCtx.name);
        _currentContext.pop();
    }
    else {
        const roleMethod = currentCtx?.roleMethodFromFunc(func);
        if (roleMethod && currentRoleMethod()?.func == func) {
            d(`Exiting RoleMethod ${roleMethod.role}.${roleMethod.method}`);
            _currentRoleMethod.pop();
        }
    }
};
const contextRules = (context, rule) => {
    rule[" FunctionDeclaration:exit"] = exitFunction;
    rule[" ArrowFunctionExpression:exit"] = exitFunction;
    rule[" FunctionDeclaration"] = (func) => enterFunction(func, context);
    rule[" ArrowFunctionExpression"] = (func) => enterFunction(func, context);
    return rule;
};
exports.contextRules = contextRules;
//# sourceMappingURL=DCIRuleHelpers.js.map