"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
const debug_1 = __importDefault(require("../debug"));
const d = (0, debug_1.default)("private-role-access");
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "private-role-access",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            Identifier(identifier) {
                const dciContext = (0, DCIRuleHelpers_1.isInContext)();
                if (!dciContext)
                    return;
                const currentRM = (0, DCIRuleHelpers_1.isInRoleMethod)();
                if (dciContext.roles.has(identifier.name)) {
                    // Check for ROLE.member access
                    if (!currentRM || currentRM.role != identifier.name) {
                        if (identifier.parent == dciContext.func) {
                            // Check if we're in the parameters of the Context, access is allowed there
                        }
                        else if (identifier.parent?.type == ast_spec_1.AST_NODE_TYPES.VariableDeclarator ||
                            identifier.parent?.type == ast_spec_1.AST_NODE_TYPES.AssignmentExpression ||
                            identifier.parent?.type == ast_spec_1.AST_NODE_TYPES.AssignmentPattern) {
                            // Check for assignments, that will be handled by the rebinding rule
                        }
                        else {
                            context.report({
                                node: identifier,
                                messageId: "externalContractCall",
                            });
                        }
                    }
                }
                else {
                    // Check for ROLE_method access
                    const rm = dciContext.roleMethodFromName(identifier.name);
                    if (!rm)
                        return;
                    if (identifier.parent?.type == ast_spec_1.AST_NODE_TYPES.VariableDeclarator) {
                        // Skip if it's not the definition itself.
                    }
                    else if (rm.isPrivate && rm.role != currentRM?.role) {
                        d(rm, { depth: 1 });
                        d(currentRM, { depth: 1 });
                        context.report({
                            node: identifier,
                            messageId: "privateCall",
                        });
                    }
                }
            },
        });
    },
    meta: {
        docs: {
            description: "The Role can only be accessed through its own RoleMethods.",
            recommended: "error",
        },
        messages: {
            privateCall: "Accessing a private RoleMethod outside its Role.",
            externalContractCall: "Accessing Role contract outside its own RoleMethods.",
        },
        type: "problem",
        schema: [],
    },
    defaultOptions: [],
});
//# sourceMappingURL=private-role-access.js.map