"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
const context_1 = require("../context");
const utils_1 = require("@typescript-eslint/utils");
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "sorted-rolemethods",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            FunctionDeclaration(node) {
                const dciContext = (0, DCIRuleHelpers_1.isContext)(node);
                if (!dciContext)
                    return;
                const roleOrder = node.body.body
                    .flatMap((s) => {
                    if (s.type == utils_1.AST_NODE_TYPES.VariableDeclaration) {
                        const roleVar = context_1.Context.potentialRoleVar(s);
                        if (!roleVar)
                            return null;
                        return roleVar.identifiers.map((r) => dciContext.roleMethodFromName(r.name) ? r.name : null);
                    }
                    else if (s.type == utils_1.AST_NODE_TYPES.FunctionDeclaration) {
                        return dciContext.roleMethodFromName(s.id.name)
                            ? s.id.name
                            : null;
                    }
                    else {
                        return null;
                    }
                })
                    .filter((s) => s);
                let current = "";
                for (const role of roleOrder) {
                    if (current > role) {
                        context.report({
                            node: dciContext.roleMethodFromName(role)
                                ?.func,
                            messageId: "unsorted",
                        });
                    }
                    else {
                        current = role;
                    }
                }
                // Make a list of Roles, check that they are in the correct order.
            },
        });
    },
    meta: {
        docs: {
            description: "RoleMethods should be sorted alphabetically.",
            recommended: "warn",
        },
        messages: {
            unsorted: "RoleMethods should be sorted alphabetically.",
        },
        type: "suggestion",
        schema: [],
    },
    defaultOptions: [],
});
//# sourceMappingURL=sorted-rolemethods.js.map